import { createRoot } from "react-dom/client";
import React from "react";
import { Menu } from "./menu";
import { createContext, useState } from "react";
import { MediaContainer } from "./media";
import { MinistreamApiClient } from "./api";

type ThemeContextType = "light" | "dark";

const ThemeContext = createContext<ThemeContextType>("light");

type AppState = {
    streamList: string[]
    selectedStream: string | undefined
}
type AppProps = {}

export class App extends React.Component<AppProps, AppState> {
    apiClient: MinistreamApiClient
    constructor(props: AppProps) {
        super(props)
        this.state = {
            streamList: [],
            selectedStream: undefined
        }
        this.apiClient = new MinistreamApiClient()
        this.updateSelect = this.updateSelect.bind(this)
    }

    async componentDidMount(): Promise<void> {
        await this.updateStreamList()
        setInterval(() => {
            this.updateStreamList()
        }, 10000)
    }

    async updateStreamList() {
        const streams = await this.apiClient.listStreams()
        const sortedStreams = streams.sort((a, b) => {
            if (a > b) {
                return -1
            } else if (a < b) {
                return 1
            }
            return 0
        })

        if (sortedStreams.length != this.state.streamList.length) {
            this.setStreamList(streams)
        }

        if (!sortedStreams.every((_, idx) => {
            return sortedStreams[idx] === this.state.streamList[idx]
        })) {
            this.setStreamList(streams)
        }
    }

    setStreamList(streamList: string[]) {
        console.log("stream list updated")
        this.setState((state) => {
            return { selectedStream: state.selectedStream, streamList: streamList }
        })
    }

    updateSelect(selected: string): void {
        if (selected !== this.state.selectedStream) {
            this.setState((state) => {
                return { streamList: state.streamList, selectedStream: selected }
            })
        }
    }

    render() {
        return (
            <>
                <Menu
                    items={this.state.streamList}
                    selectedItem={this.state.selectedStream}
                    selectCallback={this.updateSelect}
                />
                <MediaContainer selectedStream={this.state.selectedStream} api={this.apiClient} />
            </>
        )
    }
}

const rootElement = document.getElementById("app");
if (rootElement) {
    const root = createRoot(rootElement);
    root.render(<App />)
}